/*
 * attrextr (attribute extract)
 * written by Ramshankar <ramshankar@bemail.org>
 *
*/

#include <Entry.h>
#include <Node.h>
#include <Directory.h>
#include <File.h>
#include <Path.h>
#include <Mime.h>
#include <String.h>
#include <stdio.h>

BString		SizeString (off_t size);
void		DumpAttribsToFile (BDirectory dir, FILE *filePtr, bool recurse);

int main (int argc, char **argv)
{
	/* Check args */
	if (argc < 3)
	{
		printf ("%s v0.2, written by Ramshankar\n", argv[0] + 2);
		printf ("usage: %s <indir> <outfile> [-ns]\n", argv[0] + 2);
		printf ("\textracts some standard attributes from entries within a folder\n");
		printf ("\tand and dumps them to file.\n\n");
		printf (" -ns             No subdirectories (optional)\n");
		printf (" <indir>         input directory path\n");
		printf (" <outfile>       output file path\n\n");
		printf (" Report bugs to <ramshankar@bemail.org>.\n");
		return -1;
	}

	/* Things we need */
	BEntry indir;
	BDirectory infolder;
	FILE *filePtr (NULL);
	bool recurse = true;
	
	if (argc >= 4)
		if (strcmp (argv[3], "-ns") == 0)
			recurse = false;
	
	/* Some initializations */
	indir.SetTo(argv[1], true);

	/* Validity checks */
	if (indir.IsDirectory() == false)
	{
		printf ("%s is not a valid directory path", argv[1]);
		return -1;
	}

	/* Initialize necessary stuff some more checks */
	filePtr = fopen (argv[2],"w");
	if (filePtr == NULL)
	{
		printf ("error: could not initialize %s\n", argv[2]);
		return -1;
	}

	infolder.SetTo (&indir);
	if (infolder.CountEntries() == 0)
	{
		printf ("%s has no entries\n", argv[1]);
		return -1;
	}
	
	DumpAttribsToFile (infolder, filePtr, recurse);
	fclose (filePtr);
}

/*========================================================================================*/

BString SizeString (off_t v)
{
	/* Hacked & tweaked from BeShare -- thanks Jeremy! */
	char buf[256];
	if (v > (1024LL * 1024LL * 1024LL * 1024LL))
		sprintf (buf, "%.2f TB", ((double)v) / (1024LL * 1024LL * 1024LL * 1024LL));
	else if (v > (1024LL * 1024LL * 1024LL))
		sprintf(buf, "%.2f GB", ((double)v) / (1024LL * 1024LL * 1024LL));
	else if (v > (1024LL * 1024LL))
		sprintf(buf, "%.2f MB", ((double)v) / (1024LL * 1024LL));
	else if (v > (1024LL))
		sprintf(buf, "%.2f KB", ((double)v) / 1024LL);
	else
		sprintf(buf, "%Li bytes", v);
	
	BString str;
	str << buf;
	
	return str;
}

/*========================================================================================*/

void DumpAttribsToFile (BDirectory infolder, FILE *f, bool recurse)
{
	/* Recursive function to iterate entries in a folder & dump attribs to file */
	BEntry entry;
	BPath path, parentpath;
	BNode node;
	BMimeType mt;
	off_t filesize;
	char description[B_MIME_TYPE_LENGTH];
	void *buffer (NULL);
	
	while (infolder.GetNextEntry (&entry, false) == B_OK)
	{
		entry.GetPath(&path);
		if (entry.InitCheck() != B_OK)
			continue;

		path.GetParent(&parentpath);
		fprintf (f, "%s; ", parentpath.Path());
	
		node.SetTo (path.Path());
		node.ReadAttr ("BEOS:TYPE", B_STRING_TYPE, 0, &buffer, 255);
		fprintf (f, "%s; ", path.Leaf());

		mt.SetTo ((char*)&buffer);
		if (mt.GetShortDescription (description) == B_OK)
			fprintf (f, "%s; ", description);
		else
			fprintf (f, "%s; ", (char*)&buffer);
	
		entry.GetSize (&filesize);
		fprintf (f, "%s;\n", SizeString(filesize).String());

		/* Is our entry a dir? If so call this function again */
		if (entry.IsDirectory() == true && recurse == true)
		{
			BDirectory subDir (&entry);
			if (subDir.InitCheck() == B_OK)
				DumpAttribsToFile (subDir, f, true);
		}
	}
}

/*========================================================================================*/
